@@ -9,6 +9,7 @@ from matplotlib import cm
 from functools import wraps
 from numpy import AxisError
 import pickle
+from pprint import pformat
 
 
 __all__ = ('Dataset', 'Data',)
@@ -33,7 +34,6 @@ def construct_Dataset_from_dataframe(function):
 
 def _convert_ITP_to_path_to_index(index_to_path):
 	"""Convert index_to_path (``pandas.Series``) to path_to_index (``dict``) 
-	
 
 	args:
 		index_to_path (pandas.Series): index_to_path to convert
@@ -42,14 +42,7 @@ def _convert_ITP_to_path_to_index(index_to_path):
 		(dict): path_to_index, key is path and value is list of indices for that path
 
 	"""
-	path_to_index = dict()
-	for i in index_to_path.index:
-		index, path = i, index_to_path[i]
-		if path in set(path_to_index.keys()):
-			path_to_index[path].append(index)
-		else:
-			path_to_index.update({path:[index]})
-	return path_to_index
+	return {x:y.values for x,y in pd.Series(index_to_path.index, index = index_to_path.values).groupby(level=0)}
 
 def _check_file_exists(path, filename):
 	if filename in set(os.listdir(path)):
@@ -275,7 +268,7 @@ class Dataset():
 		return pd.DataFrame(self.meta_data.iloc[index]).T.reset_index(drop = True)
 
 	def _write_ekpds_file(self, filename):
-		preamble = 'pointercolumn:{}|path:{}'.format(self.pointercolumn, self.path)
+		preamble = 'pointercolumn:{}|path:{}'.format(self.pointercolumn, {key:list(self.path[key]) for key in self.path})
 		with open(filename, 'wb+') as f:
 			f.write(pickle.dumps(preamble))
 			f.write(b'########')
@@ -346,23 +339,33 @@ class Dataset():
 		"""
 		Remove references to files that do not exist in path. This may occur, for example, if you know certain data files are bad (and thus delete them from the data dir), but did not delete them while collecting data. 
 		"""
+		# determine available paths in the dataset
+		if type(self.path) == str:
+			paths = {self.path:self.meta_data.index.values}
+		else:
+			paths = self.path
+			
 		remove_index = []
-		for ind, path in enumerate(self.index_to_path):
-			if _check_file_exists(path, self.meta_data[self.pointercolumn].iloc[ind]):
-				continue
-			else:
-				remove_index.append(ind)
+			
+		for path in paths:
+			indices_to_check = paths[path]
+			existing_files = set(os.listdir(path))
+			pointercolumn_values_for_path = self.meta_data.iloc[indices_to_check][self.pointercolumn].values
+			
+			for index, value in zip(indices_to_check, pointercolumn_values_for_path):
+				if value not in existing_files:
+					remove_index.append(index)
 
 		return self.remove_index(remove_index)
 
 
-	def _group(self, by):
+	def _group(self, by, level=None):
 		"""Group data by 'by' and return a pandas dataframe. makes use of pandas.groupby
 
 		args:
 			by (str, int, label or array-like of): on what to group. 
 		"""
-		groups = self.meta_data.groupby(by = by).groups
+		groups = self.meta_data.groupby(by = by, level = level).groups
 		for ijk, key in enumerate(groups):
 			original_dataset_indices = groups[key]
 			new_row = None
@@ -561,7 +564,7 @@ class Dataset():
 				>>> data.data_keys
 				['time', 'p1', 'p2']
 
-				>>> data[0]['data']['p1'] # This corresponds to a single trial (1D data)
+				>>> data.iloc[0].p1 # This corresponds to a single trial (1D data)
 				array([  0.00898495,  0.00765674,  0.00351585, ..., -0.00679731,
 						-0.00101569, -0.00039065])
 
@@ -572,7 +575,7 @@ class Dataset():
 
 				# let's retrieve the data but with grouping
 				>>> data = dset.get_data(groupby = 'high_voltage_v')
-				>>> data[0]['data']['p1'] #vstack of all different .csv files grouped by high_voltage_v (a meta data parameter)
+				>>> data.iloc[0]['p1'] #vstack of all different .csv files grouped by high_voltage_v (a meta data parameter)
 				array([[ 0.00898495,  0.00765674,  0.00351585, ..., -0.00679731,
 						-0.00101569, -0.00039065],
 					   [-0.02172014, -0.02773615, -0.03695549, ..., -0.0203138 ,
@@ -609,15 +612,15 @@ class Dataset():
 		readfileby = self.readfileby
 
 		if type(groupby) == type(None):
-			data_to_retrieve = self._group(by = pointercolumn) #gives us a unique col for each
+			data_to_retrieve = self._group(by = None, level = 0) # gives us a unique col for each
 		else:
 			data_to_retrieve = self._group(by = groupby)
 
 		out = {}
-		for counter, i in enumerate(data_to_retrieve.index): #for each row
-			#data_to_retrieve.at[i, self.pointercolumn] is a dict
+		for counter, i in enumerate(data_to_retrieve.index): # for each row
+			# NOTE data_to_retrieve.at[i, self.pointercolumn] is a dict
 			filename_index_to_path_dict = data_to_retrieve.at[i, self.pointercolumn]
-			for k, index_of_original in enumerate(filename_index_to_path_dict): #datafile in the set of datafiles in that row
+			for k, index_of_original in enumerate(filename_index_to_path_dict):
 				try:
 					tdf = readfileby(self.index_to_path[index_of_original] + filename_index_to_path_dict[index_of_original])
 				except Exception as e:
@@ -729,7 +732,7 @@ class iDataIndexer():
 			index = i
 		return Data({index: self.indexed_dict[index]})
 
-class Data(dict):
+class Data():
 	"""Data class for maintaining and manipulating the real data. Typically retrieved via ``Dataset.get_data()``
 
 
@@ -765,8 +768,9 @@ class Data(dict):
 			}
 
 		.. code-block:: python
-			
+
 			# one can access data and/or definition as an attribute
+			
 			>>> data.definition
 			>{'frequency': {'1067hz'},
 			   'amplitude': {'500ua'},
@@ -797,8 +801,28 @@ class Data(dict):
 	"""
 
 	def __init__(self, initializer):
-		super().__init__(initializer)
+		self._dict = dict(initializer)
+		
+	def __getitem__(self, key):
+		try:
+			return self._dict.__getitem__(key)
+		except KeyError:
+			return self.__getattr__(key)
 
+	def __iter__(self):
+		return self._dict.__iter__()
+	
+	def __next__(self):
+		return self._dict.__next__()
+		
+	def __str__(self):
+		return self._dict.__str__()
+	
+	def __repr__(self):
+		return pformat(self._dict, indent=1,)
+		
+	def __len__(self):
+		return self._dict.__len__()
 
 	def __getattr__(self, key):
 		if key == 'definition' or key == 'data':            
@@ -807,24 +831,24 @@ class Data(dict):
 			return self._get_defn_attr(key)
 		except KeyError:
 			return self._get_data_attr(key)
-		
+
 	def _get_defn_attr(self, key):
 		if len(self) == 1:
-			return self[list(self.keys())[0]]['definition'][key]
-	
-		return {i:self[i]['definition'][key] for i in self}
-	
+			return self._dict[list(self._dict.keys())[0]]['definition'][key]
+
+		return {i:self._dict[i]['definition'][key] for i in self._dict}
+
 	def _get_data_attr(self, key):
 		if len(self) == 1:
-			return self[list(self.keys())[0]]['data'][key]
-	
-		return {i:self[i]['data'][key] for i in self}
-	
+			return self._dict[list(self._dict.keys())[0]]['data'][key]
+
+		return {i:self._dict[i]['data'][key] for i in self._dict}
+
 	def _get_defn_or_data(self, key):
 		if len(self) == 1:
-			return self[list(self.keys())[0]][key]
-		
-		return {i:self[i][key] for i in self}
+			return self._dict[list(self._dict.keys())[0]][key]
+
+		return {i:self._dict[i][key] for i in self._dict}
 
 	@property
 	def iloc(self):
@@ -837,21 +861,21 @@ class Data(dict):
 		example:
 
 			.. code-block:: python
-				
+
 				>>> Data.iloc[0]
-			
+
 		"""
-		return iDataIndexer(self.to_dict().copy())
+		return iDataIndexer(self._dict.copy())
 
 	@property 
 	def summary(self):
 		"""Return a summary of the definitions in data
-		
+
 		returns:
 			(dict): summary of the data included
 
 		"""
-		return _summarize_data(self.to_dict().copy())
+		return _summarize_data(self._dict.copy())
 
 	@property
 	def data_keys(self):
@@ -861,28 +885,16 @@ class Data(dict):
 		returns:
 			(list): data keys
 		"""
-		return list(self[list(self.keys())[0]]['data'].keys())
+		return list(self._dict[list(self._dict.keys())[0]]['data'].keys())
+
+	def to_ekpdat(self, file):
+		"""Save file as `.ekpdat` file.
 
-	def sort_by_definition(self, definition_key, how=None, reverse=False):
-		"""Sort Data. 
-		
 		args:
-			definition_key (key): Key to sort by.
-			how (function): Method for sorting.
-			reverse (bool): Reverse sorting
-			
-		returns:
-			(Data): Sorted Data
+			file (str): Path to file
 		"""
-		available_options = self.summary[definition_key]
-		sorted_keys = sorted(available_options, key=how, reverse=reverse)
-		
-		out = {}
-		
-		for i, key in enumerate(sorted_keys):
-			out.update({i: self.contains({definition_key:[key]})[0]})
-		
-		return Data(out)
+		with open(file, 'wb') as f:
+			pickle.dump(self._dict, f)
 
 	def _get_indices_satisfying_definition_condtion(self, condition):
 		"""need docstring"""
@@ -896,15 +908,15 @@ class Data(dict):
 		indices = [] #to hold which data satisfies condition
 
 		for condition_key in condition:
-			for index in self:
-				defn = self[index]['definition']
+			for index in self._dict:
+				defn = self._dict[index]['definition']
 				if _check_definition_contains_or(defn, condition_key, condition[condition_key]):
 					indices.append(index)
 		return indices
 
 	def filter(self, data_condition_function_dict, definition_condition_dict=None, additional_data_keys_to_filter=None):
 		"""Filter the data.
-		
+
 		args:
 			data_condition_function_dict (dict): a dict with one entry. key is data key, value is function which operates on a single value.
 			definition_condition_dict (dict): a dict specifying specific definition conditions, which when satisfied can allow their data to be operated on. If None, all data will be filtered.
@@ -916,25 +928,25 @@ class Data(dict):
 		Examples:
 
 			Filter the data such that the data key 'R' only contains values >10. This leaves all other data keys unchanged.
-			
+
 			.. code-block:: python
 
 				>>> Data.filter({'R': lambda x: x>10}) 
-			
+
 
 
 			Filter data corresponding to an amplitude of '500ua' such that data key 'R' contains only values >10.
-			
+
 			.. code-block:: python
 
 				>>> Data.filter({'R': lambda x: x>10}, {'amplitude':'500ua'}) 
-	
+
 			Filter data based on 'saturation' but also filter the 'switching_time' data.
-		
+
 			.. code-block:: python
 
 				>>> Data.filter({'saturation': lambda x: x > .003}, additional_data_keys_to_filter='switching_time')
-	
+
 
 			You may use this method to remove outliers, for example.
 
@@ -946,18 +958,18 @@ class Data(dict):
 			additional_data_keys_to_filter = []
 		else:
 			additional_data_keys_to_filter = np.array([additional_data_keys_to_filter]).flatten()
-		
+
 		assert len(data_condition_function_dict) <= 1, "more than one condition supplied in data_condition_function_dict. only one is supported at a time."
 		data_function_key = list(data_condition_function_dict)[0]
 		assert hasattr(data_condition_function_dict[data_function_key], '__call__'), "value associated with key '{}' in data_condition_function_dict is a not a function. is type {}".format(data_condition_function_dict, type(data_condition_function_dict[data_function_key]))
-		assert data_function_key in set(self[0]['data'].keys()), "data_function_key '{}' not in data. available keys are {}".format(data_function_key, set(self[0]['data'].keys()))
-		
+		assert data_function_key in set(self._dict[0]['data'].keys()), "data_function_key '{}' not in data. available keys are {}".format(data_function_key, set(self[0]['data'].keys()))
+
 		sat_indices = self._get_indices_satisfying_definition_condtion(definition_condition_dict)
 		if definition_condition_dict == {}:
-			sat_indices = set(self.keys())
-		all_index = set(self.keys())
+			sat_indices = set(self._dict.keys())
+		all_index = set(self._dict.keys())
 		not_sat_indices = all_index - set(sat_indices)
-		
+
 		func = data_condition_function_dict[data_function_key]
 
 		keys_to_update = [key for key in additional_data_keys_to_filter]
@@ -965,18 +977,18 @@ class Data(dict):
 		keys_to_update.append(data_function_key)
 		keys_to_update = keys_to_update[::-1]
 
-		
+
 		for index in sat_indices:
-			checker = self[index]['data'][data_function_key]
+			checker = self._dict[index]['data'][data_function_key]
 			for key in keys_to_update:
-				old = self[index]['data'][key]
+				old = self._dict[index]['data'][key]
 				old_shape = old.shape
 				#ensure that all keys in keys_to_update have the same dimensionality
 				if old_shape != checker.shape:
 					raise ValueError('data corresponding to data_key "{}" does not have the same shape as data corresponding to data_function_key "{}"'.format(key, data_function_key))
-				self[index]['data'].update({key:old[func(checker)]})
-			
-		return Data(self.to_dict())
+				self._dict[index]['data'].update({key:old[func(checker)]})
+
+		return Data(self._dict)
 
 	def contains(self, condition):
 		"""Returns data specified by condition.
@@ -986,11 +998,11 @@ class Data(dict):
 
 		returns:
 			(Data): data satisfying condition 
-		
+
 		examples:
 
 			.. code-block:: python
-				
+
 				#Providing multiple values will be joined by logical or, i.e.
 				Data.contains({'high_voltage_v':{'100mv', '200mv'}})
 				#will search for all data with '100mv' OR '200mv' for high_voltage_v. 
@@ -1000,7 +1012,7 @@ class Data(dict):
 				#will search for x = 1 AND y = 2.
 
 
-	
+
 		"""
 		indices = self._get_indices_satisfying_definition_condtion(condition)
 
@@ -1019,27 +1031,27 @@ class Data(dict):
 		#build the final Data:
 		out = {}
 		for new, old in enumerate(indices_out):
-			out.update({new: self[old]})
+			out.update({new: self._dict[old]})
 
 		return Data(out)
 
 
-	def mean(self, inplace = False):
+	def mean(self, inplace=False):
 		"""
 		Return data class where data is averaged across 0th axis. This is best used for averaging across trials or similar.
 
 		args:
-			inplace (bool): do operation inplace
+			inplace (bool): Do operation inplace
 
 		returns:
 			(Data): averaged data across 0th axis.
 		"""
-		
+
 		if not inplace:
 			tmp_out = {}
-			for key in self:
-				tmp_out.update({key:self[key].copy()})
-				data = self[key]['data']
+			for key in self._dict:
+				tmp_out.update({key:self._dict[key].copy()})
+				data = self._dict[key]['data']
 				mean_data = {}
 				for k in data:
 					if len(data[k].shape) == 1: #1d data
@@ -1063,8 +1075,8 @@ class Data(dict):
 			(numpy.array): Concatenated array of all data corresponding to data_key for all indices in self. 
 
 		"""
-		for ijk, i in enumerate(self):
-			current = self[i]['data'][data_key].flatten()
+		for ijk, i in enumerate(self._dict):
+			current = self._dict[i]['data'][data_key].flatten()
 			if ijk == 0:
 				out = current
 				continue
@@ -1073,10 +1085,10 @@ class Data(dict):
 		return out
 
 	def apply(self, function_on_data, pass_defn=False, ignore_errors=True,**kwargs):
-		"""Apply data_function to the data in each index. ``**kwargs`` will be passed to data_function.
+		"""Apply data_function to the data in each index. ``**kwargs`` will be passed to data_function. If function_on_data returns 'None', that piece of data will be dropped. 
 
 		args:
-			function_on_data (function): f(dict) -> dict. Function is passed the data_dict (corresponding to self[index]['data']).
+			function_on_data (function): f(dict) -> dict. Function is passed the data_dict for each index.
 			pass_defn (bool): Whether or not to pass the definition to function_on_data. If True, will be passed with other kwargs. 
 			ignore_errors (bool): If True, errors in function_on_data will be printed, but not raised. Resulting data will be original data. If False, errors will be raised.
 
@@ -1086,7 +1098,7 @@ class Data(dict):
 		examples:
 
 			.. code-block:: python
-				
+
 				>>> some_data
 				>
 				{
@@ -1126,35 +1138,39 @@ class Data(dict):
 		"""
 
 		data_function = function_on_data
-		tmp_out = self.to_dict().copy()
+		tmp_out = {}
+		new_key = 0
 
-		for key in tmp_out.keys():
+		for key in self._dict.keys():
 			try:
 
 				if pass_defn:
 					#ensure no overlap between passed arguments and definition arguments:
-					overlap = set(kwargs.keys()).intersection(set(tmp_out[key]['definition'].keys()))
+					overlap = set(kwargs.keys()).intersection(set(self._dict[key]['definition'].keys()))
 					if len(overlap) != 0:
 						raise ValueError('kwargs passed in both definition and as kwargs in .apply(). Overlapping keys are "{}"'.format(overlap))
 
-					to_pass = tmp_out[key]['definition'].copy()
+					to_pass = self._dict[key]['definition'].copy()
 					for key in kwargs:
 						to_pass.update({key:kwargs})
 					internal_out = {
-						'definition':tmp_out[key]['definition'],
-						'data':data_function(tmp_out[key]['data'].copy(), defn = tmp_out[key]['definition'], **to_pass)
+						'definition':self._dict[key]['definition'],
+						'data':data_function(self._dict[key]['data'].copy(), defn = self._dict[key]['definition'], **to_pass)
 					}
 				else:
 					internal_out = {
-						'definition':tmp_out[key]['definition'],
-						'data':data_function(tmp_out[key]['data'].copy(), **kwargs)
+						'definition':self._dict[key]['definition'],
+						'data':data_function(self._dict[key]['data'].copy(), **kwargs)
 					}
+				if internal_out['data'] == 'None':
+					continue
 
-				tmp_out.update({key:internal_out})
+				tmp_out.update({new_key:internal_out})
+				new_key += 1
 			except Exception as e:
 				if ignore_errors:
 					print('Error in data_function: {} \n{}'.format(data_function.__name__, e))
-					print('Skipping data key: {} with defintion: \n{}'.format(key, tmp_out[key]['definition']))
+					print('Skipping data key: {} with defintion: \n{}'.format(key, self._dict[key]['definition']))
 				else:
 					raise e
 
@@ -1167,7 +1183,7 @@ class Data(dict):
 
 		returns:
 			(dict): a dict class with identical structure"""
-		return {key: self[key] for key in self.keys()}
+		return self._dict
 
 	def plot(self, x=None, y=None, ax=None, color = None, cmap = 'viridis', labelby = None, **kwargs):
 		"""
@@ -1195,38 +1211,38 @@ class Data(dict):
 			raise KeyError('cmap "{}" not supported. Supported colormaps are {}'.format(cmap, cm.cmaps_listed.keys()))  
 
 		if color == None:
-			colors = [cm.cmaps_listed[cmap](x) for x in np.linspace(0, 1, len(self.keys()))]
+			colors = [cm.cmaps_listed[cmap](x) for x in np.linspace(0, 1, len(self._dict.keys()))]
 		else: 
-			colors = [color for i in range(len(self.keys()))]
+			colors = [color for i in range(len(self._dict.keys()))]
 
-		for color, index in zip(colors, self.keys()):
+		for color, index in zip(colors, self._dict.keys()):
 			if x == None:
-				data_keys_to_plot = list(self[index]['data'].keys())
+				data_keys_to_plot = list(self._dict[index]['data'].keys())
 			else:
-				xs = self[index]['data'][x]
-				data_keys_to_plot = set(self[index]['data'].keys()) - set({x})
+				xs = self._dict[index]['data'][x]
+				data_keys_to_plot = set(self._dict[index]['data'].keys()) - set({x})
 
 			if labelby == None:
 				label = None
 			else:
-				label = self[index]['definition'][labelby]
+				label = self._dict[index]['definition'][labelby]
 
 			if type(y) == type(None):
 				pass
 			else:
 				data_keys_to_plot = set(np.array([y]).flatten())
-				
+
 
 			for plotkey in data_keys_to_plot:
-				to_plot = self[index]['data'][plotkey]
-				
+				to_plot = self._dict[index]['data'][plotkey]
+
 				if len(to_plot.shape) == 1: #1d data
 					if x == None:
 						ax.plot(to_plot, color = color, label = label, **kwargs)
 					else:
 						ax.plot(xs, to_plot, color = color, label = label, **kwargs)
 					continue
-					
+
 				for i in range(to_plot.shape[0]):
 					if i == 0: #label only the first becuase we don't want a big legend with many trials
 						if x == None:
@@ -1273,38 +1289,38 @@ class Data(dict):
 			raise KeyError('cmap "{}" not supported. Supported colormaps are {}'.format(cmap, cm.cmaps_listed.keys()))  
 
 		if color == None:
-			colors = [cm.cmaps_listed[cmap](x) for x in np.linspace(0, 1, len(self.keys()))]
+			colors = [cm.cmaps_listed[cmap](x) for x in np.linspace(0, 1, len(self._dict.keys()))]
 		else: 
-			colors = [color for i in range(len(self.keys()))]
+			colors = [color for i in range(len(self._dict.keys()))]
 
-		for color, index in zip(colors, self.keys()):
+		for color, index in zip(colors, self._dict.keys()):
 			if x == None:
-				data_keys_to_plot = list(self[index]['data'].keys())
+				data_keys_to_plot = list(self._dict[index]['data'].keys())
 			else:
-				xs = self[index]['data'][x]
-				data_keys_to_plot = set(self[index]['data'].keys()) - set({x})
+				xs = self._dict[index]['data'][x]
+				data_keys_to_plot = set(self._dict[index]['data'].keys()) - set({x})
 
 			if labelby == None:
 				label = None
 			else:
-				label = self[index]['definition'][labelby]
+				label = self._dict[index]['definition'][labelby]
 
 			if type(y) == type(None):
 				pass
 			else:
 				data_keys_to_plot = set(np.array([y]).flatten())
-				
+
 
 			for plotkey in data_keys_to_plot:
-				to_plot = self[index]['data'][plotkey]
-				
+				to_plot = self._dict[index]['data'][plotkey]
+
 				if len(to_plot.shape) == 1: #1d data
 					if x == None:
 						ax.scatter(to_plot, color = color, label = label, **kwargs)
 					else:
 						ax.scatter(xs, to_plot, color = color, label = label, **kwargs)
 					continue
-					
+
 				for i in range(to_plot.shape[0]):
 					if i == 0: #label only the first becuase we don't want a big legend with many trials
 						if x == None:
@@ -1324,3 +1340,166 @@ class Data(dict):
 			return fig, ax
 		else:
 			return ax
+
+	def sort(self, by, key=None, reverse=False):
+		"""
+		Sort `Data` by definition key. This might be useful, for example in plotting with color maps. This Method sorts Data over mulitple indices, does not sort Data within an index. 
+
+		args:
+			by (str or key): Definition key. The definition key that you are sorting on must be unique for each index in your `Data` object. 
+			key (function): Method for accessing value to sort.
+			reverse (bool): Reverse order
+
+		returns:
+			(Data): Sorted Data
+
+		examples:
+
+			Sort Data on parameter 'test'
+
+			.. code-block:: python
+
+				>>> data = dset.get_data()
+				>>> data.summary
+				> 
+				{'param': {'v'},
+				 'test': {0, 1, 2, 3, 4},
+				 'test2': {'100mv', '15mv', '1mv', '20mv', '32mv'}}
+
+				# Sort by definition key 'test'
+				>>> data = data.sort(by='test')
+
+				# Confirm sorted:
+				>>> data.test
+				> {0: {0}, 1: {1}, 2: {2}, 3: {3}, 4: {4}}
+
+				### Sort in reverse order:
+
+				>>> data = dset.get_data()
+				>>> data.sort(by = 'test', reverse = True).test
+				> {0: {4}, 1: {3}, 2: {2}, 3: {1}, 4: {0}}
+
+			
+			Sort Data on a key that is not int or float:
+
+			.. code-block:: python
+
+				>>> data = dset.get_data()
+				>>> data.summary
+				> 
+				{'param': {'v'},
+				 'test': {0, 1, 2, 3, 4},
+				 'test2': {'100mv', '15mv', '1mv', '20mv', '32mv'}}
+				
+				# unsorted data
+				>>> data.test2
+				> {0: {'1mv'}, 1: {'100mv'}, 2: {'20mv'}, 3: {'15mv'}, 4: {'32mv'}}
+
+				# Sort by definition key 'test2'
+				>>> sort_by = lambda x: float(x.replace('mv', '')) # get rid of 'mv' suffix and convert to float
+				>>> data = data.sort(by='test2', key=sort_by)
+				>>> data.test2
+				> {0: {'1mv'}, 1: {'15mv'}, 2: {'20mv'}, 3: {'32mv'}, 4: {'100mv'}}
+
+		"""
+		sorter = _data_sorter(self._dict, by).sort(key=key, reverse=reverse)
+		sorted_out = {}
+		for i in sorter.mapper:
+			sorted_out.update({i:self._dict[sorter.mapper[i]]})
+			
+		return Data(sorted_out)
+	
+class _data_sorter():
+	"""Class for sorting data. Upon running `self.sort` self contains two attributes, `.values` and `.value_index_mapper`. `values` contains the sorted values corresponding to arg `_definition_key` and `value_index_mappper` is a dict mapping each (sorted) value to a `Data` index. 
+		
+		args:
+			_dict (dict): Dict representation of Data object. 
+			_definition_key (str or key): Definition key to sort on.
+
+		examples:
+
+			One can explicitly use the _data_sorter class
+
+			.. code-block:: python 
+
+				>>> data = dset.get_data()
+				>>> data.summary
+				> {'test':{0,1,2,3,4}}
+
+				# Create a sorter for definition key 'test' 
+				>>> sorter = _data_sorter(data._dict, 'test')
+				>>> sorter.sort()
+				>>> sorted_out = {}
+				>>> for i in sorter.mapper:
+						sorted_out.update({i:data._dict[sorter.mapper[i]]})
+				>>> Data(sorted_out)
+				> 
+				{0: {'data': {'R': array([1, 2, 3], dtype=int64)},
+					 'test': {0}}},
+				 1: {'data': {'R': array([1, 2, 3], dtype=int64)},
+					 'test': {1}}},
+				 2: {'data': {'R': array([1, 2, 3], dtype=int64)},
+					 'test': {2}}},
+				 3: {'data': {'R': array([1, 2, 3], dtype=int64)},
+					 'test': {3}}},
+				 4: {'data': {'R': array([1, 2, 3], dtype=int64)},
+					 'test': {4}}}}
+
+			The same exact functionality can be accessed with `Data.sort()`
+
+			.. code-block:: python
+
+				>>> data = dset.get_data()
+				>>> test.summary
+				> {'data':{0,1,2,3,4}}
+
+				# Sort by definition key 'test'
+				>>> data.sort(by='test')
+				> 
+				{0: {'data': {'R': array([1, 2, 3], dtype=int64)},
+					 'test': {0}}},
+				 1: {'data': {'R': array([1, 2, 3], dtype=int64)},
+					 'test': {1}}},
+				 2: {'data': {'R': array([1, 2, 3], dtype=int64)},
+					 'test': {2}}},
+				 3: {'data': {'R': array([1, 2, 3], dtype=int64)},
+					 'test': {3}}},
+				 4: {'data': {'R': array([1, 2, 3], dtype=int64)},
+					 'test': {4}}}}
+
+
+		"""
+	
+	def __init__(self, _dict, _definition_key):
+		self._dict = _dict
+		self._definition_key = _definition_key
+		self._keys = list(_dict.keys())
+		
+	def sort(self, key=None, reverse=False):
+		"""
+		Populate attributes `.values` and `value_index_mapper`. 
+
+		args:
+			key (function): Method for accessing value to sort. 
+			reverse (bool): Reverse order
+
+		returns:
+			(_data_sorter) 
+		"""
+		values = []
+		if type(key) == type(None):
+			key = lambda x: x
+
+		for i in self._keys:
+			definition_value = self._dict[i]['definition'][self._definition_key]
+			assert len(definition_value) == 1, "Definition for index '{}' is not unique. It contains multiple values: {}. Cannot sort.".format(i, definition_value)
+			value = list(definition_value)[0]
+			values.append(value)	
+
+		float_values = [key(value) for value in values]
+		argsort = np.argsort(float_values)
+		if reverse:
+			argsort = argsort[::-1]
+		self.mapper = {i:old_index for i, old_index in enumerate(argsort)}
+			
+		return self
